% read file
fid = fopen('./InfluenzaData.txt', 'r');
C = textscan(fid, '%d %d %d %s', 'delimiter', '\t', 'CollectOutput', true);
fclose(fid);

% partition data
% strainData: (1 - H1N1, 2 - BVictoria, 3 - BYamagata)

strains = unique(C{2}); % strains contain the names of the strains

strainData = cell(1,3)  % strainData is a cell with 3 entries, each entry
                        % containing the data matrix for the strain

index1 = cellfun(@(x) isequal(x, strains{1}), C{2});
strainData{1} = C{1}(index1,:);
index2 = cellfun(@(x) isequal(x, strains{2}), C{2});
strainData{2} = C{1}(index2,:);
index3 = cellfun(@(x) isequal(x, strains{3}), C{2});
strainData{3} = C{1}(index3,:);

% compute mean and median virus loads, pooled and separately

% mmParams is a matrix with the following rows:
% |time|mean|median|standard deviation|# of elements (at that time)|
mmParams_0 = means(C{1});
mmParams_1 = means(strainData{1});
mmParams_2 = means(strainData{2});
mmParams_3 = means(strainData{3});

% constant for easily changing whether we estimate the pooled data
% logarithmically (using the linearlized model function) or not.
LOG_PLOT = true;
names = cell(1,2);
names{1} = 'Mean';
names{2} = 'Median';

names2 = cell(1,4);
names2{1} = 'Mean';
names2{2} = 'Median';
names2{3} = 'Mean (log)';
names2{4} = 'Median (log)';

% estimate and plot for pooled data (all strains)
% weights are currently: Mean/Number of elements.
weights0_mean = mmParams_0(:,2)./mmParams_0(:,5);
weights0_median = mmParams_0(:,3)./mmParams_0(:,5);
est0_mean = getEstimates(mmParams_0, weights0_mean, false);
est0_median = getEstimates(mmParams_0, weights0_median, false);
est0_mean2 = getEstimates(mmParams_0, weights0_mean, true);
est0_median2 = getEstimates(mmParams_0, weights0_median, true);
est0 = [est0_mean; est0_median; est0_mean2; est0_median2];
plotEstimates(mmParams_0, est0, names2, 'All strains');

% estimate and plot for pooled data (strain 1 - H1N1 resistant)
weights1_mean = mmParams_1(:,2)./mmParams_1(:,5);
weights1_median = mmParams_1(:,3)./mmParams_1(:,5);
est1_mean = getEstimates(mmParams_1, weights1_mean, false);
est1_median = getEstimates(mmParams_1, weights1_median, false);
est1_mean2 = getEstimates(mmParams_1, weights1_mean, true);
est1_median2 = getEstimates(mmParams_1, weights1_median, true);
est1 = [est1_mean; est1_median; est1_mean2; est1_median2];
plotEstimates(mmParams_1, est1, names2, strains(1));

% estimate and plot for pooled data (strain 2 - H1N1 sensitive)
weights2_mean = mmParams_2(:,2)./mmParams_2(:,5);
weights2_median = mmParams_2(:,3)./mmParams_2(:,5);
est2_mean = getEstimates(mmParams_2, weights2_mean, false);
est2_median = getEstimates(mmParams_2, weights2_median, false);
est2_mean2 = getEstimates(mmParams_2, weights2_mean, true);
est2_median2 = getEstimates(mmParams_2, weights2_median, true);
est2 = [est2_mean; est2_median; est2_mean2; est2_median2];
plotEstimates(mmParams_2, est2, names2, strains(2));

% estimate and plot for pooled data (strain 3 - B sensitive)
weights3_mean = mmParams_3(:,2)./mmParams_3(:,5);
weights3_median = mmParams_3(:,3)./mmParams_3(:,5);
est3_mean = getEstimates(mmParams_3, weights3_mean, false);
est3_median = getEstimates(mmParams_3, weights3_median, false);
est3_mean2 = getEstimates(mmParams_3, weights3_mean, true);
est3_median2 = getEstimates(mmParams_3, weights3_median, true);
est3 = [est3_mean; est3_median; est3_mean2; est3_median2];
plotEstimates(mmParams_3, est3, names2, strains(3));

% analyze and estimate the data seperately for each patient
p = C{1}(end,1);
patientData = cell(1,p);
patientEst = cell(1,p);
meanEst = [];
patientStrains = cell(1,p);

% if statement so that we can disable this code fragment
if true
    % for all patients, apply the fit and visualize it
    % (each strain has its own figure - we end up with 3 figures)
    for i=1:p
        index = i == C{1}(:,1);
        patientData{i} = C{1}(index,:);
        patientStrains{i} = unique(C{2}(index));

        % change the figure, depending on what strain the patient is
        if isequal(patientStrains{i}, strains(1))
            figure(11);
            title(strains(1));
            hold all;
        end
        if isequal(patientStrains{i}, strains(2))
            figure(12);
            title(strains(2));
            hold all;
        end
        if isequal(patientStrains{i}, strains(3))
            figure(13);
            title(strains(3));
            hold all;
        end

        xlabel('Days after treatment begin')
        ylabel('Virus load [1/ml solute]')

        % get the parameters of the data for this patient
        patParams = means(patientData{i})
        
        % use proportional weights in this case (i.e. V(i,t) itself)
        weights5 = patParams(:,2);
        p_est = getEstimates(patParams, weights5, false);
        patientEst{i} = p_est;      % record estimate
        meanEst = [meanEst; p_est]; % 

        % plot the patient into its figure
        % (we plot both original data and the fit, in different styles
        plot(patientData{i}(:,2), patientData{i}(:,3), 'b--o');
        hold on
        plot(patientData{i}(:,2), lindecay(p_est, double(patientData{i}(:,2))), 'r-.o');
        set(gca,'YScale','log')
    end
end

median(meanEst);

% get the entries of the parameter estimates for each strain
pat_index1 = cellfun(@(x) isequal(x, strains(1)), patientStrains);
pat_index2 = cellfun(@(x) isequal(x, strains(2)), patientStrains);
pat_index3 = cellfun(@(x) isequal(x, strains(3)), patientStrains);

lin_inv = @(param) (log(param(1)/10)*(1/param(2)));

tmp1 = cellfun(lin_inv, patientEst(pat_index1));
tmp2 = cellfun(lin_inv, patientEst(pat_index2));
tmp3 = cellfun(lin_inv, patientEst(pat_index3));

m1 = median(tmp1);
m2 = median(tmp2);
m3 = median(tmp3);

cl1 = cellfun(@(p) (p(2)), patientEst(pat_index1));
cl2 = cellfun(@(p) (p(2)), patientEst(pat_index2));
cl3 = cellfun(@(p) (p(2)), patientEst(pat_index3));

cl_m1 = median(cl1);
cl_m2 = median(cl2);
cl_m3 = median(cl3);

% plot clearance distribution
figure(19)
hold all;
axis([0 4 -0.5 3])
set(gca,'XTick',[1,2,3])
set(gca,'XTickLabel',{'H1N1 resistant','H1N1 sensitive','B sensitive'})
plot(repmat(1,1,length(cl1))+(-1+2*rand(1,length(cl1)))/4,cl1, '*')
plot(repmat(2,1,length(cl2))+(-1+2*rand(1,length(cl2)))/4,cl2, 'square')
plot(repmat(3,1,length(cl3))+(-1+2*rand(1,length(cl3)))/4,cl3, 'diamond')
% draw a line for the median value
plot([0.75 1.25], [cl_m1 cl_m1], 'Color', 'black');
plot([1.75 2.25], [cl_m2 cl_m2], 'Color', 'black');
plot([2.75 3.25], [cl_m3 cl_m3], 'Color', 'black');
ylabel('Virus Clearance [1/day]');

% plot time to eradication
figure(20)
hold all;
axis([0 4 0 30])
set(gca,'XTick',[1,2,3])
set(gca,'XTickLabel',{'H1N1 resistant','H1N1 sensitive','B sensitive'})
plot(repmat(1,1,length(tmp1))+(-1+2*rand(1,length(tmp1)))/4,tmp1, '*')
plot(repmat(2,1,length(tmp2))+(-1+2*rand(1,length(tmp2)))/4,tmp2, 'square')
plot(repmat(3,1,length(tmp3))+(-1+2*rand(1,length(tmp3)))/4,tmp3, 'diamond')
% draw a line for the median value
plot([0.75 1.25], [m1 m1], 'Color', 'black');
plot([1.75 2.25], [m2 m2], 'Color', 'black');
plot([2.75 3.25], [m3 m3], 'Color', 'black');
ylabel('Time to eradication [days]');